/*
 * Hash table API.
 */

#ifndef _HASH_TABLE_H
#define _HASH_TABLE_H

/* Interface to user supplied hash function which returns key for arbitrary data. */
typedef unsigned int (*hash_callback_t)(void *data);
typedef int boolean_t;

/* Interface to user supplied compare function for arbitrary data. */
typedef int (*hash_compare_t)(const void *a, const void *b);

struct hash_table {
	struct hash_bucket *table;
	size_t count;
	size_t collisions;
	size_t size;
	hash_compare_t compare;
	hash_callback_t hash;
};

struct hash_bucket {
	struct hash_bucket *next;
	void *key;
	void *value;
};

struct hash_table_iter {
	struct hash_table *table;
	unsigned int index;
	struct hash_bucket *bucket;
};

/* Create new hash table. */
struct hash_table *hash_table_new(size_t size_hint, hash_callback_t hash, hash_compare_t keycmp);

/* Destroy hash table and all it's resources. */
void hash_table_destroy(struct hash_table **table);

/* Insert key with data to hash table. */
int hash_table_insert(struct hash_table *table, void *key, void *data);

/* Replace existing data within hash table. */
boolean_t hash_table_replace(struct hash_table *table, void *key, void *data);

/* Remove key from hash table. */
boolean_t hash_table_remove(struct hash_table *table, void *key);

/* Prepare iterator context. */
struct hash_table_iter *hash_table_iterate_init(struct hash_table *table);

/* Iterate through hash table key/data pairs. */
boolean_t hash_table_iterate(struct hash_table_iter *iter, void **res_key, void **res_data);

/* Deinit iterator context. */
void hash_table_iterate_deinit(struct hash_table_iter **iter);

#endif
